package com.erikloyer.util
{
	import flash.geom.Point;
	
	public class Geometry
	{
		static public var LINES_COINCIDENT:String = "linesCoincident";
		static public var LINES_PARALLEL:String = "linesParallel";
		static public var LINES_INTERSECT:String = "linesIntersect";
		static public var SEGMENTS_INTERSECT:String = "segmentsIntersect";
		static public var SEGMENT_A_INTERSECTS:String = "segmentAIntersects";
		static public var SEGMENT_B_INTERSECTS:String = "segmentBIntersects";
		
		/**
		 * Converts radians to degrees.
		 * 
		 * @param	r	Value in radians.
		 */
		static public function getDegreesFromRadians(r:Number):Number {
			var degrees:Number = (r / (Math.PI * 2)) * 360;
			return degrees;
		}
		
		/**
		 * Converts degrees to radians.
		 * 
		 * @param	d	Value in degrees.
		 */
		static public function getRadiansFromDegrees(d:Number):Number {
			var radians:Number = (d / 360) * (Math.PI * 2);
			return radians;
		}
		
		/**
		 * Returns the angle between the two points in radians.
		 * 
		 * @param	p1		The first point.
		 * @param	p2		The second point.
		 */
		static public function getAngleBetweenPoints(p1:Point, p2:Point):Number {
			var xdiff:Number = p2.x - p1.x;
			var ydiff:Number = p2.y - p1.y;
			var angle:Number = Math.atan2(ydiff, xdiff);
			return angle;
		}
		
		/**
		 * Returns an object describing the intersection between two line segments.
		 * 
		 * Code adapted from an algorithm by Paul Bourke (http://local.wasp.uwa.edu.au/~pbourke/geometry/lineline2d/)
		 * 
		 * @param	p1		Starting point of the first line segment.
		 * @param	p2		Ending point of the first line segment.
		 * @param	p3		Starting point of the second line segment.
		 * @param	p4		Ending point of the second line segment.
		 */
		static public function getIntersectionBetweenLines(p1:Point, p2:Point, p3:Point, p4:Point):Object {
			
			var numeratorA:Number = (p4.x - p3.x) * (p1.y - p3.y) - (p4.y - p3.y) * (p1.x - p3.x);
			var numeratorB:Number = (p2.x - p1.x) * (p1.y - p3.y) - (p2.y - p1.y) * (p1.x - p3.x);
			var denominator:Number = (p4.y - p3.y) * (p2.x - p1.x) - (p4.x - p3.x) * (p2.y - p1.y);
			
			var infoObj:Object = {result:"", loc:null};
			
			if ((numeratorA == 0) && (numeratorB == 0) && (denominator == 0)) {
				infoObj.result = Geometry.LINES_COINCIDENT;
			} else if (denominator == 0) {
				infoObj.result = Geometry.LINES_PARALLEL;
			} else {
				var uA:Number = numeratorA / denominator;
				var uB:Number = numeratorB / denominator;
				infoObj.loc = new Point;
				infoObj.loc.x = p1.x + uA * (p2.x - p1.x);
				infoObj.loc.y = p1.y + uA * (p2.y - p1.y);
				if ((uA >= 0) && (uA <= 1)) {
					if ((uB >= 0) && (uB <= 1)) {
						infoObj.result = Geometry.SEGMENTS_INTERSECT;
					} else {
						infoObj.result = Geometry.SEGMENT_A_INTERSECTS;
					}
				} else if ((uB >= 0) && (uB <= 1)) {
					infoObj.result = Geometry.SEGMENT_B_INTERSECTS;
				} else {
					infoObj.result = Geometry.LINES_INTERSECT;
				}
			}
			
			return infoObj;
		}
		
		/**
		 * Returns true if point p is inside the triangle described by points a, b and c.
		 * 
		 * Code adapted from an algorithm by Graeme McRae (http://mcraefamily.com/MathHelp/GeometryPointAndTriangle2.htm)
		 * 
		 * @param	p	The point to test.
		 * @param	a	The first vertex of the triangle.
		 * @param	b	The second vertex of the triangle.
		 * @param	c	The third vertex of the triangle.
		 */
		static public function pointInTriangle(p:Point, a:Point, b:Point, c:Point):Boolean {
			var result:Boolean = false;
			var fAB:Number = (p.y - a.y) * (b.x - a.x) - (p.x - a.x) * (b.y - a.y);
			var fCA:Number = (p.y - c.y) * (a.x - c.x) - (p.x - c.x) * (a.y - c.y);
			var fBC:Number = (p.y - b.y) * (c.x - b.x) - (p.x - b.x) * (c.y - b.y);
			if (((fAB * fBC) > 0) && ((fBC * fCA) > 0)) {
				result = true;
			}
			return result;
		}
		
		/**
		 * Returns the index to the point in the given array which is closest to the point p.
		 * 
		 * @param	p	The point to test.
		 * @param	a	The array of points to text.
		 */
		static public function getClosestPointIndex(p:Point, a:Array):Number {
			var n:Number = a.length;
			var result:Number = -1;
			var closestDistance:Number = 99999;
			var distance:Number;
			for (var i:Number=0; i<n; i++) {
				distance = Point.distance(p, a[i]);
				if (distance < closestDistance) {
					closestDistance = distance;
					result = i;
				}
			}
			return result;
		}
		
		/**
		 * Returns an information object about the triangle formed by the given points.
		 * 
		 * @param	pA	The first point of the triangle.
		 * @param	pB	The second point of the triangle.
		 * @param	pC	The third point of the triangle.
		 */
		static public function getTriangleInfo(pA:Point, pB:Point, pC:Point):Object {
			var result:Object;
			var A:Number = Point.distance(pB, pC);
			var B:Number = Point.distance(pA, pB);
			var C:Number = Point.distance(pA, pC);
			var sides:Array = new Array(A, B, C);
			var longestSide:Number = 0;
			var longestSideIndex:Number;
			for (var i:Number=0; i<3; i++) {
				if (sides[i] > longestSide) {
					longestSide = sides[i];
					longestSideIndex = i;
				}
			}
			sides.splice(longestSideIndex, 1);
			var largestAngle:Number = Math.acos((Math.pow(sides[0], 2) + Math.pow(sides[1], 2) - Math.pow(longestSide, 2)) / (2 * sides[0] * sides[1]));
			var nextSide:Number = sides.pop();
			var secondAngle:Number = Math.asin((nextSide * Math.sin(largestAngle)) / longestSide);
			var thirdAngle:Number = (Math.PI * 2) - (largestAngle + secondAngle);
			var angles:Array = new Array(largestAngle, secondAngle, thirdAngle);
			angles.sort(Array.NUMERIC);
			sides = new Array(A, B, C);
			sides.sort(Array.NUMERIC);
			result = {angles:angles, largestAngle:largestAngle, sides:sides, longestSide:longestSide};
			if (largestAngle > (Math.PI / 2)) {
				result.type = "obtuse";
			} else if (largestAngle == (Math.PI / 2)) {
				result.type = "right";
			} else if (largestAngle == secondAngle == thirdAngle) {
				result.type = "equilateral";
			} else {
				result.type = "acute";
			}
			return result;
		}
		 
		
	}
}